/*******************************************************************************
* Copyright 2009 Seapine Software, Inc. All rights reserved.
*
* IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
* in consideration of your agreement to the following terms, and your use, installation,
* modification or redistribution of this Software constitutes acceptance of these terms.
* If you do not agree with these terms, please do not use, install, modify or redistribute
* this Software.
*
* In consideration of your agreement to abide by the following terms, and subject to these
* terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
* this original Seapine software (the Software), to use, reproduce, modify and redistribute
* the Software, with or without modifications, in source and/or binary forms; provided that
* if you redistribute the Software in its entirety and without modifications, you must retain
* this notice and the following text and disclaimers in all such redistributions of the Software.
* Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
* or promote products derived from the Software without specific prior written permission from
* Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
* implied, are granted by Seapine herein, including but not limited to any patent rights that
* may be infringed by your derivative works or by other works in which the Software may be
* incorporated.
*
* The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
* EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
* OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
*
* IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
* REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
* UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
* IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/
using System;
using System.Collections;
using System.IO;
using System.Text;

namespace Seapine_Reports_Dashboard
{
	public class CGIEngine
	{
      public string lastError() { return (m_strLastError); }
      protected string m_strLastError = "";

      protected ttsoapcgi m_engine = null;
      protected long m_lSession = 0;

      protected string m_strUsername = "";
      protected string m_strPassword = "";

      protected Hashtable m_hashTableColumns = new Hashtable();

      protected CGIEngine() { m_engine = new ttsoapcgi(); m_engine.Timeout = 100000; }

      public CGIEngine(string p_strUrl, string p_strUsr, string p_strPwd)
      {
         m_engine = new ttsoapcgi();
         m_engine.Timeout = 100000;
         m_engine.Url = p_strUrl;

         m_strUsername = p_strUsr;
         m_strPassword = p_strPwd;
      }

      public CFile[] runReport(string p_strProject, long p_lReportId)
      {
         CFile[] af = null;
         try
         {
            if (login(p_strProject))
            {
               CReportRunResults results = m_engine.getReportRunResultsByRecordID(m_lSession, p_lReportId);
               af = results.mFileList;
            }
         }
         catch (Exception p_exception) { m_strLastError = "runReport: " + p_exception.Message; }
         finally { logout(); }
         return (af);
      }

      public CFile[] runReport(string p_strProject, string p_strReportName)
      {
         CFile[] af = null;
         try
         {
            if (login(p_strProject))
            {
               CReportRunResults results = m_engine.getReportRunResultsByName(m_lSession, p_strReportName);
               af = results.mFileList;
            }
         }
         catch (Exception p_exception) { m_strLastError = "runReport: " + p_exception.Message; }
         finally { logout(); }
         return (af);
      }

      public ArrayList fetchReports(string p_strProject)
      {
         ArrayList list = new ArrayList();
         try
         {
            if (login(p_strProject))
            {
               // -harpm_DEBUG CTableColumn[] atc2 = m_engine.getColumnsForTable(m_lSession, "Report");
               CTableColumn[] atc = new CTableColumn[5];
               atc[0] = new CTableColumn(); atc[0].name = "Record ID";
               atc[1] = new CTableColumn(); atc[1].name = "Name";
               atc[2] = new CTableColumn(); atc[2].name = "Title";
               atc[3] = new CTableColumn(); atc[3].name = "Type";
               atc[4] = new CTableColumn(); atc[4].name = "Contains";

               CRecordListSoap reclist = m_engine.getRecordListForTable( m_lSession, "Report", "", atc);
               if (reclist != null && reclist.records != null && reclist.records.Length > 0)
               {
                  for (int i = 0; i < reclist.records.Length; ++i)
                  {
                     string strTableName = reclist.records[i].row[4].value.TrimEnd('s');
                     TTReport report = new TTReport(getColumnsForTable(strTableName), Convert.ToInt32(reclist.records[i].row[0].value), reclist.records[i].row[1].value, reclist.records[i].row[2].value, reclist.records[i].row[3].value, reclist.records[i].row[4].value);
                     list.Add(report);
                  }
               }
            }
         }
         catch (Exception p_exception) { m_strLastError = "fetchReports: " + p_exception.Message; }
         finally { logout(); }
         return (list);
      }

      public virtual ArrayList fetchProjects()
      {
         ArrayList list = new ArrayList();
         try
         {
            CProject[] aproject = m_engine.getProjectList(m_strUsername, m_strPassword);
            for (int i = 0; i < aproject.Length; ++i)
               list.Add(aproject[i].database.name);
         }
         catch (Exception p_exception) { m_strLastError = "fetchProjects: " + p_exception.Message; }
         return (list);
      }

      public bool validateConnection(string p_strUrl, string p_strUsername, string p_strPassword, ref string p_strError)
      {
         bool fResult = false;
         try
         {
            m_engine.Url = p_strUrl;
            CProject[] aproject = m_engine.getProjectList(p_strUsername, p_strPassword);
            if (aproject.Length == 0) { p_strError = "Connection succeeded, but your username does not have permission to see any projects."; }
            else { fResult = true; }
         }
         catch (Exception p_exception) { p_strError = "TestTrack connection failed: " + p_exception.Message; }
         return (fResult);
      }

      protected CTableColumn[] getColumnsForTable(string p_strTableName)
      {
         if (p_strTableName == null || p_strTableName.Length == 0 || p_strTableName == "Custom") { return (null); }
         CTableColumn[] atc = null;
         try
         {
            atc = (CTableColumn[])m_hashTableColumns[p_strTableName];
            if (atc == null)
               m_hashTableColumns.Add(p_strTableName, m_engine.getColumnsForTable(m_lSession, p_strTableName));
            atc = (CTableColumn[])m_hashTableColumns[p_strTableName];
         }
         catch (Exception) {}
         return (atc);
      }

      protected virtual bool login(string p_strProject)
      {
         try
         {
            CProject project = null;
            CProject[] aproject = m_engine.getProjectList(m_strUsername, m_strPassword);
            for (int i = 0; i < aproject.Length && project == null; ++i)
            {
               if (aproject[i].database.name == p_strProject) { project = aproject[i]; }
            }

            m_lSession = m_engine.ProjectLogon(project, m_strUsername, m_strPassword);
         }
         catch (Exception p_exception)
         {
            m_strLastError = "login: " + p_exception.Message;
            m_lSession = 0;
         }
         return (m_lSession > 0);
      }

      protected bool logout()
      {
         try
         {
            if (m_lSession > 0) { m_engine.DatabaseLogoff(m_lSession); }
         }
         catch (Exception p_exception) { m_strLastError = "logout: " + p_exception.Message; }
         finally { m_lSession = 0; }
         return (true);
      }
   }
}
