/*******************************************************************************
* Copyright 2009 Seapine Software, Inc. All rights reserved.
*
* IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
* in consideration of your agreement to the following terms, and your use, installation,
* modification or redistribution of this Software constitutes acceptance of these terms.
* If you do not agree with these terms, please do not use, install, modify or redistribute
* this Software.
*
* In consideration of your agreement to abide by the following terms, and subject to these
* terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
* this original Seapine software (the Software), to use, reproduce, modify and redistribute
* the Software, with or without modifications, in source and/or binary forms; provided that
* if you redistribute the Software in its entirety and without modifications, you must retain
* this notice and the following text and disclaimers in all such redistributions of the Software.
* Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
* or promote products derived from the Software without specific prior written permission from
* Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
* implied, are granted by Seapine herein, including but not limited to any patent rights that
* may be infringed by your derivative works or by other works in which the Software may be
* incorporated.
*
* The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
* EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
* OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
*
* IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
* REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
* UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
* IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/
using System;
using System.Collections.Generic;
using System.Text;

namespace Seapine_Reports_Dashboard
{
   class TTReport : IComparable
   {
      public enum TTReportType { rtUnknown = 0, rtDetail, rtList, rtDistribution, rtTrend }
      public enum TTReportContains { rcUnknown = 0, rcDefect, rcTestCase, rcTestRun, rcRequirement, rcRequirementDocument, rcUser, rcCustomer, rcSecurityGroup, rcFolder, rcTestConfig }

      private const string c_strUknown = "<unknown>";

      private const string c_strRTDetail = "Detail";
      private const string c_strRTList = "List";
      private const string c_strRTDistribution = "Distribution";
      private const string c_strRTTrend = "Trend";

      private const string c_strRCDefects = "Defects";
      private const string c_strRCTestCases = "Test Cases";
      private const string c_strRCTestRuns = "Test Runs";
      private const string c_strRCRequirements = "Requirements";
      private const string c_strRCRequirementDocuments = "Requirement Documents";
      private const string c_strRCUsers = "Users";
      private const string c_strRCCustomers = "Customers";
      private const string c_strRCSecurityGroups = "Security Groups";
      private const string c_strRCFolders = "Folders";
      private const string c_strRCTestConfigs = "Test Configs";

      protected long m_lId = 0;
      protected string m_strName = "";
      protected string m_strTitle = "";
      protected TTReportType m_rt = TTReportType.rtUnknown;
      protected TTReportContains m_rc = TTReportContains.rcUnknown;

      public TTReport(CTableColumn[] p_atc, long p_lId, string p_strName, string p_strTitle, string p_strType, string p_strContains)
      {
         Id = p_lId;
         Name = p_strName;
         Title = p_strTitle;
         setType(p_strType);
         setContains(ref p_atc, p_strContains);
      }

      public long Id { get { return (m_lId); } set { m_lId = value; } }
      public string Name { get { return (m_strName); } set { m_strName = value; } }
      public string Title { get { return (m_strTitle); } set { m_strTitle = value; } }

      public TTReportType getType() { return (m_rt); }
      public TTReportContains getContains() { return (m_rc); }

      public string getTypeAsString()
      {
         switch (m_rt)
         {
            case TTReportType.rtDetail: return (c_strRTDetail);
            case TTReportType.rtList: return (c_strRTList);
            case TTReportType.rtDistribution: return (c_strRTDistribution);
            case TTReportType.rtTrend: return (c_strRTTrend);
         }
         return (c_strUknown);
      }

      public string getContainsAsString()
      {
         switch (m_rc)
         {
            case TTReportContains.rcDefect: return (c_strRCDefects);
            case TTReportContains.rcTestCase: return (c_strRCTestCases);
            case TTReportContains.rcTestRun: return (c_strRCTestRuns);
            case TTReportContains.rcRequirement: return (c_strRCRequirements);
            case TTReportContains.rcRequirementDocument: return (c_strRCRequirementDocuments);
            case TTReportContains.rcUser: return (c_strRCUsers);
            case TTReportContains.rcCustomer: return (c_strRCCustomers);
            case TTReportContains.rcSecurityGroup: return (c_strRCSecurityGroups);
            case TTReportContains.rcFolder: return (c_strRCFolders);
            case TTReportContains.rcTestConfig: return (c_strRCTestConfigs);
         }
         return (c_strUknown);
      }

      public void setType(TTReportType p_rt) { m_rt = p_rt; }
      public void setContains(TTReportContains p_rc) { m_rc = p_rc; }

      protected void setType(string p_str)
      {
         if (p_str == null || p_str.Length == 0)
            setType(TTReportType.rtUnknown);
         else if (p_str == c_strRTDetail)
            setType(TTReportType.rtDetail);
         else if (p_str == c_strRTList)
            setType(TTReportType.rtList);
         else if (p_str == c_strRTDistribution)
            setType(TTReportType.rtDistribution);
         else if (p_str == c_strRTTrend)
            setType(TTReportType.rtTrend);
         else
            setType(TTReportType.rtUnknown);
      }

      public int CompareTo(object obj)
      {  // sort by Name, although I could see Title also being a valid way to sort.
         TTReport report = (TTReport)obj;
         if (this.Name == report.Name) { return (this.Id.CompareTo(report.Id)); } // Sort by Id, if primary key matches.
         else { return (this.Name.CompareTo(report.Name)); }
      }

      protected void setContains(ref CTableColumn[] p_atc, string p_str)
      {
         if (p_atc == null || p_atc.Length == 0 || p_str == null || p_str.Length == 0)
            setContains(TTReportContains.rcUnknown);
         else if (p_str == c_strRCDefects || p_str.Equals(Properties.Settings.Default.Label_Defects, StringComparison.CurrentCultureIgnoreCase))
            setContains(TTReportContains.rcDefect);
         else if (p_str == c_strRCTestCases || p_str.Equals(Properties.Settings.Default.Label_TestCases, StringComparison.CurrentCultureIgnoreCase))
            setContains(TTReportContains.rcTestCase);
         else if (p_str == c_strRCTestRuns || p_str.Equals(Properties.Settings.Default.Label_TestRuns, StringComparison.CurrentCultureIgnoreCase))
            setContains(TTReportContains.rcTestRun);
         else if (p_str == c_strRCRequirements || p_str.Equals(Properties.Settings.Default.Label_Requirements, StringComparison.CurrentCultureIgnoreCase))
            setContains(TTReportContains.rcRequirement);
         else if (p_str == c_strRCRequirementDocuments || p_str.Equals(Properties.Settings.Default.Label_RequirementDocuments, StringComparison.CurrentCultureIgnoreCase))
            setContains(TTReportContains.rcRequirementDocument);
         else if (p_str == c_strRCUsers)
            setContains(TTReportContains.rcUser);
         else if (p_str == c_strRCCustomers)
            setContains(TTReportContains.rcCustomer);
         else if (p_str == c_strRCSecurityGroups)
            setContains(TTReportContains.rcSecurityGroup);
         else if (p_str == c_strRCFolders)
            setContains(TTReportContains.rcFolder);
         else if (p_str == c_strRCTestConfigs)
            setContains(TTReportContains.rcTestConfig);
         else
         {  // they've renamed the field label & haven't given us the new label, so we have to get creative since the API provides no call to figure this out directly.
            setContains(TTReportContains.rcUnknown);
            for (int i = 0; i < p_atc.Length && getContains() == TTReportContains.rcUnknown; ++i)
            {  // these search strings should be unique to the object type we're assuming.
               if (p_atc[i].name == "Number Reported") { setContains(TTReportContains.rcDefect); }
               else if (p_atc[i].name.Contains("Requirement Type")) { setContains(TTReportContains.rcRequirement); }
               else if (p_atc[i].name.Contains("Number of ")) { setContains(TTReportContains.rcTestCase); }
               else if (p_atc[i].name == "Has Run Results") { setContains(TTReportContains.rcTestRun); }
            }
         }
      }
   }
}
