﻿/*******************************************************************************
* Copyright 2009 Seapine Software, Inc. All rights reserved.
*
* IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (“Seapine”)
* in consideration of your agreement to the following terms, and your use, installation,
* modification or redistribution of this Software constitutes acceptance of these terms.
* If you do not agree with these terms, please do not use, install, modify or redistribute
* this Software.
*
* In consideration of your agreement to abide by the following terms, and subject to these
* terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
* this original Seapine software (the “Software”), to use, reproduce, modify and redistribute
* the Software, with or without modifications, in source and/or binary forms; provided that
* if you redistribute the Software in its entirety and without modifications, you must retain
* this notice and the following text and disclaimers in all such redistributions of the Software.
* Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
* or promote products derived from the Software without specific prior written permission from
* Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
* implied, are granted by Seapine herein, including but not limited to any patent rights that
* may be infringed by your derivative works or by other works in which the Software may be
* incorporated.
*
* The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
* EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
* OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
*
* IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
* REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
* UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
* IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/

using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;

namespace Seapine_Reports_Dashboard
{
   public class Harp
   {
      protected LinkButton m_btn = null;
      public LinkButton Name { get { return m_btn; } set { m_btn = value; } }
   }

   public partial class _Default : System.Web.UI.Page
   {
      protected CGIEngine m_engine = null;

      protected string m_strUrl = Properties.Settings.Default.Url;
      protected string m_strUsr = Properties.Settings.Default.Usr;
      protected string m_strPwd = Properties.Settings.Default.Pwd;

      protected string m_strCachePath = "temp";

      protected void Page_Load(object sender, EventArgs e)
      {
         statusmsg.Text = ""; // start fresh!
         m_engine = new CGIEngine(m_strUrl, m_strUsr, m_strPwd); // is null on every refresh?
         if (ddProjects.Items.Count > 0) { return; } // don't redo everything!

         ddProjects.DataSource = m_engine.fetchProjects();
         if (ddProjects.DataSource != null && ((ArrayList)ddProjects.DataSource).Count > 0)
            ddProjects.DataBind();
         else
            statusmsg.Text = m_engine.lastError();

         populateReportTree(ddProjects.Text);
         cleanCache();

         // set icon labels, if they've been set by the user.
         if (Properties.Settings.Default.Label_Defects.Length > 0) lDefectIcon.Text = Properties.Settings.Default.Label_Defects;
         if (Properties.Settings.Default.Label_TestCases.Length > 0) lTestCaseIcon.Text = Properties.Settings.Default.Label_TestCases;
         if (Properties.Settings.Default.Label_TestRuns.Length > 0) lTestRunIcon.Text = Properties.Settings.Default.Label_TestRuns;
         if (Properties.Settings.Default.Label_Requirements.Length > 0) lRequirementIcon.Text = Properties.Settings.Default.Label_Requirements;
         if (Properties.Settings.Default.Label_RequirementDocuments.Length > 0) lRequirementDocumentIcon.Text = Properties.Settings.Default.Label_RequirementDocuments;
      }

      protected void populateReportTree(string p_strProject)
      {
         try
         {
            ArrayList listDefects = new ArrayList();
            ArrayList listTCs = new ArrayList();
            ArrayList listTRs = new ArrayList();
            ArrayList listRQs = new ArrayList();
            ArrayList listRDs = new ArrayList();

            ArrayList list = m_engine.fetchReports(p_strProject);
            for (int i = 0; i < list.Count; ++i)
            {
               TTReport report = (TTReport)list[i];
               if (report == null) { continue; }
               else if (report.getContains() == TTReport.TTReportContains.rcDefect)
                  listDefects.Add(report);
               else if (report.getContains() == TTReport.TTReportContains.rcTestCase)
                  listTCs.Add(report);
               else if (report.getContains() == TTReport.TTReportContains.rcTestRun)
                  listTRs.Add(report);
               else if (report.getContains() == TTReport.TTReportContains.rcRequirement)
                  listRQs.Add(report);
               else if (report.getContains() == TTReport.TTReportContains.rcRequirementDocument)
                  listRDs.Add(report);
            }

            listDefects.Sort();
            repeaterDefects.DataSource = listDefects;
            repeaterDefects.DataBind();
            tlhdrdefects.Style.Add("display", listDefects.Count > 0 ? "":"none");

            listTCs.Sort();
            repeaterTestCases.DataSource = listTCs;
            repeaterTestCases.DataBind();
            tlhdrtestcases.Style.Add("display", listTCs.Count > 0 ? "" : "none");

            listTRs.Sort();
            repeaterTestRuns.DataSource = listTRs;
            repeaterTestRuns.DataBind();
            tlhdrtestruns.Style.Add("display", listTRs.Count > 0 ? "" : "none");

            listRQs.Sort();
            repeaterRequirements.DataSource = listRQs;
            repeaterRequirements.DataBind();
            tlhdrrequirements.Style.Add("display", listRQs.Count > 0 ? "" : "none");

            listRDs.Sort();
            repeaterRequirementDocuments.DataSource = listRDs;
            repeaterRequirementDocuments.DataBind();
            tlhdrrequirementdocuments.Style.Add("display", listRDs.Count > 0 ? "" : "none");
         }
         catch (Exception p_exception) { Response.Write(p_exception.Message); }
      }

      protected void cleanCache()
      {  // delete reports older than 1 day.
         try
         {
            string[] astr = System.IO.Directory.GetFiles(HttpContext.Current.Server.MapPath(m_strCachePath));
            foreach (string str in astr)
            {
               System.IO.FileInfo fi = new System.IO.FileInfo(str);
               if (fi.CreationTime < DateTime.Now.AddDays(-1)) // is it 1 day old?
               {
                  try { fi.Delete(); } catch {} // if it doesn't delete just move on.
               }
            }
         }
         catch (Exception) {}
      }
   }
}
